@@ -4,6 +4,7 @@ import collections
 import json
 import os
 import shlex
+import shutil
 import signal
 from pathlib import Path, PurePath
 from subprocess import Popen
@@ -17,7 +18,8 @@ from werkzeug.utils import _filename_ascii_strip_re
 
 from sapporo.const import RUN_DIR_STRUCTURE, RUN_DIR_STRUCTURE_KEYS
 from sapporo.model import RunRequest, State
-from sapporo.model.factory import generate_service_info
+from sapporo.model.factory import (generate_executable_workflows,
+                                   generate_service_info)
 from sapporo.model.sapporo_wes_1_0_1 import AttachedFile
 
 
@@ -114,7 +116,6 @@ def secure_filepath(filepath: str) -> Path:
         node = "_".join(node.split())
         # Change [^A-Za-z0-9_.-] to empty.
         node = str(_filename_ascii_strip_re.sub("", node))
-        node = node.strip("._")
         if node not in ["", ".", ".."]:
             nodes.append(node)
 
@@ -156,11 +157,22 @@ def prepare_run_dir(run_id: str, run_request: RunRequest) -> None:
     write_file(run_id, "wf_engine_params",
                convert_wf_engine_params_str(run_request))
 
+    write_file(run_id, "service_info", generate_service_info())
+    write_file(run_id, "executable_workflows", generate_executable_workflows())
+    with current_app.config["RUN_SH"].open(mode="r", encoding="utf-8") as f:
+        run_sh_content = f.read()
+    write_file(run_id, "run_sh", run_sh_content)
+
+    yevis_metadata = request.form.get("yevis_metadata", None)
+    if yevis_metadata is not None:
+        write_file(run_id, "yevis_metadata", yevis_metadata)
+
     write_workflow_attachment(run_id, run_request)
 
 
 def dump_sapporo_config() -> Dict[str, Any]:
     return {
+        "sapporo_version": current_app.config["SAPPORO_VERSION"],
         "get_runs": current_app.config["GET_RUNS"],
         "workflow_attachment": current_app.config["WORKFLOW_ATTACHMENT"],
         "registered_only_mode": current_app.config["REGISTERED_ONLY_MODE"],
@@ -197,20 +209,6 @@ def convert_wf_engine_params_str(run_request: RunRequest) -> str:
 
 def write_workflow_attachment(run_id: str, run_request: RunRequest) -> None:
     exe_dir = resolve_content_path(run_id, "exe_dir")
-    endpoint = sapporo_endpoint()
-    wf_attachment_obj: List[AttachedFile] = json.loads(
-        run_request["workflow_attachment"] or "[]")
-    for file in wf_attachment_obj:
-        name = file["file_name"]
-        url = file["file_url"]
-        parsed_url = parse.urlparse(url)
-        if parsed_url.scheme in ["http", "https"] and not url.startswith(endpoint):
-            file_path = exe_dir.joinpath(secure_filepath(name)).resolve()
-            file_path.parent.mkdir(parents=True, exist_ok=True)
-            response = requests.get(url)
-            with file_path.open(mode="wb") as f:
-                f.write(response.content)
-
     if current_app.config["WORKFLOW_ATTACHMENT"]:
         workflow_attachment = request.files.getlist("workflow_attachment")
         for file_storage in workflow_attachment:
@@ -221,6 +219,32 @@ def write_workflow_attachment(run_id: str, run_request: RunRequest) -> None:
                 file_storage.save(file_path)
 
 
+# Called in run.sh
+def download_workflow_attachment(inputted_run_dir: str) -> None:
+    run_dir: Path = Path(inputted_run_dir).resolve()
+    config_path = run_dir.joinpath(RUN_DIR_STRUCTURE["sapporo_config"])
+    with config_path.open(mode="r", encoding="utf-8") as f:
+        sapporo_config = json.load(f)
+    endpoint = sapporo_config["sapporo_endpoint"]
+    exe_dir = run_dir.joinpath(RUN_DIR_STRUCTURE["exe_dir"])
+    run_request_path = run_dir.joinpath(RUN_DIR_STRUCTURE["run_request"])
+    with run_request_path.open(mode="r", encoding="utf-8") as f:
+        run_request = json.load(f)
+    wf_attachment_obj: List[AttachedFile] = json.loads(run_request["workflow_attachment"] or "[]")
+    for file in wf_attachment_obj:
+        name = file["file_name"]
+        url = file["file_url"]
+        parsed_url = parse.urlparse(url)
+        if parsed_url.scheme in ["http", "https"] and not url.startswith(endpoint):
+            file_path = exe_dir.joinpath(secure_filepath(name)).resolve()
+            file_path.parent.mkdir(parents=True, exist_ok=True)
+            with requests.get(url, stream=True) as res:
+                if res.status_code == 200:
+                    with file_path.open(mode="wb") as f:
+                        res.raw.decode_content = True
+                        shutil.copyfileobj(res.raw, f, 1024 * 1024)
+
+
 def fork_run(run_id: str) -> None:
     run_dir: Path = resolve_run_dir_path(run_id)
     stdout: Path = resolve_content_path(run_id, "stdout")
@@ -276,6 +300,7 @@ def path_hierarchy(original_path: Path, dir_path: Path) -> Any:
     return hierarchy
 
 
+# Called in run.sh
 def dump_outputs_list(inputted_run_dir: str) -> None:
     run_dir: Path = Path(inputted_run_dir).resolve()
     run_id = run_dir.name
@@ -289,7 +314,7 @@ def dump_outputs_list(inputted_run_dir: str) -> None:
     outputs = []
     for output_file in output_files:
         outputs.append({
-            "file_name": output_file.name,
+            "file_name": str(output_file.relative_to(outdir_path)),
             "file_url":
                 f"{base_remote_url}{str(output_file.relative_to(run_dir))}"
         })
