@@ -1,7 +1,9 @@
 #!/usr/bin/env bash
-set -eu
+set -e
 
 function run_wf() {
+  echo "INITIALIZING" >${state}
+  download_workflow_attachment
   echo "RUNNING" >${state}
   date +"%Y-%m-%dT%H:%M:%S" >${start_time}
   # e.g. when wf_engine_name=cwltool, call function run_cwltool
@@ -16,26 +18,22 @@ function run_wf() {
   date +"%Y-%m-%dT%H:%M:%S" >${end_time}
   echo 0 >${exit_code}
   echo "COMPLETE" >${state}
+  generate_ro_crate &
+  clean_rundir &
   exit 0
 }
 
 function run_cwltool() {
-  local container="quay.io/commonwl/cwltool:3.1.20211107152837"
+  local container="quay.io/commonwl/cwltool:3.1.20220628170238"
   local cmd_txt="${DOCKER_CMD} ${container} --outdir ${outputs_dir} ${wf_engine_params} ${wf_url} ${wf_params} 1>${stdout} 2>${stderr}"
   echo ${cmd_txt} >${cmd}
   eval ${cmd_txt} || executor_error
 }
 
 function run_nextflow() {
-  local container="nextflow/nextflow:21.10.5"
-  local cmd_txt=""
-  if [[ $(jq 'select(.outdir) != null' ${wf_params}) ]]; then
-    # It has outdir as params.
-    cmd_txt="docker run -i --rm ${D_SOCK} -v ${run_dir}:${run_dir} -w=${exe_dir} ${container} nextflow -dockerize run ${wf_url} ${wf_engine_params} -params-file ${wf_params} --outdir ${outputs_dir} 1>${stdout} 2>${stderr}"
-  else
-    # It has NOT outdir as params.
-    cmd_txt="docker run -i --rm ${D_SOCK} -v ${run_dir}:${run_dir} -w=${exe_dir} ${container} nextflow -dockerize run ${wf_url} ${wf_engine_params} -params-file ${wf_params} -work-dir ${outputs_dir} 1>${stdout} 2>${stderr}"
-  fi
+  local container="nextflow/nextflow:22.04.4"
+  local cmd_txt="docker run --rm ${D_SOCK} -v ${run_dir}:${run_dir} -w=${exe_dir} ${container} nextflow -dockerize run ${wf_url} ${wf_engine_params} -params-file ${wf_params} --outdir ${outputs_dir} -work-dir ${exe_dir} 1>${stdout} 2>${stderr}"
+  find ${exe_dir} -type f -exec chmod 777 {} \;
   echo ${cmd_txt} >${cmd}
   eval ${cmd_txt} || executor_error
 }
@@ -48,19 +46,19 @@ function run_toil() {
 }
 
 function run_cromwell() {
-  local container="broadinstitute/cromwell:72"
+  local container="broadinstitute/cromwell:80"
   local wf_type=$(jq -r ".workflow_type" ${run_request})
   local wf_type_version=$(jq -r ".workflow_type_version" ${run_request})
-  local cmd_txt="docker run -i --rm ${D_SOCK} -v ${run_dir}:${run_dir} -v /tmp:/tmp -v /usr/bin/docker:/usr/bin/docker -w=${exe_dir} ${container} run ${wf_engine_params} ${wf_url} -i ${wf_params} -m ${exe_dir}/metadata.json --type ${wf_type} --type-version ${wf_type_version} 1>${stdout} 2>${stderr}"
+  local cmd_txt="docker run --rm ${D_SOCK} -v ${run_dir}:${run_dir} -v /tmp:/tmp -v /usr/bin/docker:/usr/bin/docker -w=${exe_dir} ${container} run ${wf_engine_params} ${wf_url} -i ${wf_params} -m ${exe_dir}/metadata.json --type ${wf_type} --type-version ${wf_type_version} 1>${stdout} 2>${stderr}"
   echo ${cmd_txt} >${cmd}
   eval ${cmd_txt} || executor_error
   if [[ ${wf_type} == "CWL" ]]; then
     jq -r ".outputs[].location" "${exe_dir}/metadata.json" | while read output_file; do
-      cp ${output_file} ${outputs_dir}/
+      cp ${output_file} ${outputs_dir}/ || true
     done
   elif [[ ${wf_type} == "WDL" ]]; then
     jq -r ".outputs | to_entries[] | .value" "${exe_dir}/metadata.json" | while read output_file; do
-      cp ${output_file} ${outputs_dir}/
+      cp ${output_file} ${outputs_dir}/ || true
     done
   fi
 }
@@ -79,7 +77,7 @@ function run_snakemake() {
     fi
   fi
   local wf_basedir="$(dirname ${wf_url_local})"
-  # NOTE this are common conventions but not hard requirements for Snakemake Standardized Usage.
+  # NOTE these are common conventions but not hard requirements for Snakemake Standardized Usage.
   local wf_schemas_dir="${wf_basedir}/schemas"
   local wf_scripts_dir="${wf_basedir}/scripts"
   local wf_results_dir="${wf_basedir}/results"
@@ -88,12 +86,12 @@ function run_snakemake() {
     chmod a+x "${wf_scripts_dir}/"*
   fi
 
-  local container="snakemake/snakemake:v6.9.1"
-  local cmd_txt="docker run -i --rm -v ${run_dir}:${run_dir} -w=${exe_dir} ${container} snakemake ${wf_engine_params} --configfile ${wf_params} --snakefile ${wf_url_local} 1>${stdout} 2>${stderr}"
+  local container="snakemake/snakemake:v7.8.3"
+  local cmd_txt="docker run --rm -v ${run_dir}:${run_dir} -w=${exe_dir} ${container} snakemake ${wf_engine_params} --configfile ${wf_params} --snakefile ${wf_url_local} 1>${stdout} 2>${stderr}"
   echo ${cmd_txt} >${cmd}
   eval ${cmd_txt} || executor_error
 
-  docker run -i --rm -v ${run_dir}:${run_dir} -w=${exe_dir} ${container} \
+  docker run --rm -v ${run_dir}:${run_dir} -w=${exe_dir} ${container} \
     snakemake --configfile ${wf_params} --snakefile ${wf_url_local} --summary 2>/dev/null | tail -n +2 | cut -f 1 |
     while read file_path; do
       dir_path=$(dirname ${file_path})
@@ -109,6 +107,25 @@ function run_ep3() {
   eval ${cmd_txt} || executor_error
 }
 
+function run_streamflow() {
+  if [[ "${wf_url}" == http://* ]] || [[ "${wf_url}" == https://* ]]; then
+    # It is a remote file.
+    local wf_url_local="${exe_dir}/$(basename ${wf_url})"
+    curl -fsSL -o ${wf_url_local} ${wf_url} || executor_error
+  else
+    # It is a local file.
+    if [[ "${wf_url}" == /* ]]; then
+      local wf_url_local="${wf_url}"
+    else
+      local wf_url_local="${exe_dir}/${wf_url}"
+    fi
+  fi
+  local container="alphaunito/streamflow:0.1.3-base"
+  local cmd_txt="docker run --mount type=bind,source=${run_dir},target=/streamflow/project --mount type=bind,source=${outputs_dir},target=/streamflow/results ${container} run /streamflow/project/exe/$(basename ${wf_url_local}) 1>${stdout} 2>${stderr}"
+  echo ${cmd_txt} >${cmd}
+  eval ${cmd_txt} || executor_error
+}
+
 function cancel() {
   # Pre-cancellation procedures
   if [[ ${wf_engine_name} == "cwltool" ]]; then
@@ -121,10 +138,18 @@ function cancel_cwltool() {
   :
 }
 
+function download_workflow_attachment() {
+  python3 -c "from sapporo.run import download_workflow_attachment; download_workflow_attachment('${run_dir}')" || executor_error
+}
+
 function generate_outputs_list() {
   python3 -c "from sapporo.run import dump_outputs_list; dump_outputs_list('${run_dir}')" || executor_error
 }
 
+function generate_ro_crate() {
+  python3 -c "from sapporo.ro_crate import generate_ro_crate; generate_ro_crate('${run_dir}')" || "{}" >"${run_dir}/ro-crate-metadata.json" || true
+}
+
 function upload() {
   local protocol=$(jq -r '.tags | fromjson | .export_output.protocol' ${run_request})
   case ${protocol} in
@@ -162,6 +187,15 @@ aws --endpoint-url ${endpoint} s3 cp ${outputs_dir} s3://${bucket_name}/${dirnam
   eval ${cmd_txt} || uploader_error
 }
 
+function clean_rundir() {
+  # Find files under run_dir older than env integer SAPPORO_DATA_REMOVE_OLDER_THAN_DAYS and delete them in a background process
+  local re_pattern="^[0-9]+$"
+  local base_run_dir=$(realpath ${run_dir}/../..)
+  if [[ ! -z ${SAPPORO_DATA_REMOVE_OLDER_THAN_DAYS} ]] && [[ ${SAPPORO_DATA_REMOVE_OLDER_THAN_DAYS} =~ ${re_pattern} ]]; then
+    find ${base_run_dir} -mindepth 2 -maxdepth 2 -mtime "+${SAPPORO_DATA_REMOVE_OLDER_THAN_DAYS}" -type d -exec rm -r {} \; >/dev/null 2>&1 &
+  fi
+}
+
 # ==============================================================
 # If you are not familiar with sapporo, please don't edit below.
 
@@ -191,7 +225,7 @@ wf_engine_params=$(head -n 1 ${wf_engine_params_file})
 # Sibling docker command
 D_SOCK="-v /var/run/docker.sock:/var/run/docker.sock"
 D_TMP="-v /tmp:/tmp"
-DOCKER_CMD="docker run -i --rm ${D_SOCK} -e DOCKER_HOST=unix:///var/run/docker.sock ${D_TMP} -v ${run_dir}:${run_dir} -w=${exe_dir}"
+DOCKER_CMD="docker run --rm ${D_SOCK} -e DOCKER_HOST=unix:///var/run/docker.sock ${D_TMP} -v ${run_dir}:${run_dir} -w=${exe_dir}"
 
 # 4 Exit cases
 # 1. The description of run.sh was wrong.
@@ -205,6 +239,8 @@ function desc_error() {
   echo ${original_exit_code} >${exit_code}
   date +"%Y-%m-%dT%H:%M:%S" >${end_time}
   echo "SYSTEM_ERROR" >${state}
+  generate_ro_crate &
+  clean_rundir &
   exit ${original_exit_code}
 }
 
@@ -214,6 +250,8 @@ function executor_error() {
   echo ${original_exit_code} >${exit_code}
   date +"%Y-%m-%dT%H:%M:%S" >${end_time}
   echo "EXECUTOR_ERROR" >${state}
+  generate_ro_crate &
+  clean_rundir &
   exit ${original_exit_code}
 }
 
@@ -223,6 +261,8 @@ function uploader_error() {
   echo ${original_exit_code} >${exit_code}
   date +"%Y-%m-%dT%H:%M:%S" >${end_time}
   echo "UPLOADER_ERROR" >${state}
+  generate_ro_crate &
+  clean_rundir &
   exit ${original_exit_code}
 }
 
@@ -241,6 +281,8 @@ function kill_by_system() {
   echo ${original_exit_code} >${exit_code}
   date +"%Y-%m-%dT%H:%M:%S" >${end_time}
   echo "SYSTEM_ERROR" >${state}
+  generate_ro_crate &
+  clean_rundir &
   exit ${original_exit_code}
 }
 
@@ -250,6 +292,8 @@ function cancel_by_request() {
   echo ${original_exit_code} >${exit_code}
   date +"%Y-%m-%dT%H:%M:%S" >${end_time}
   echo "CANCELED" >${state}
+  generate_ro_crate &
+  clean_rundir &
   exit ${original_exit_code}
 }
 
